/* ***********************************************************
 * This file was automatically generated on 2012-10-12.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for controlling two relays
	/// </summary>
	public class BrickletDualRelay : Device 
	{
		private static byte FUNCTION_SET_STATE = 1;
		private static byte FUNCTION_GET_STATE = 2;
		private static byte FUNCTION_SET_MONOFLOP = 3;
		private static byte FUNCTION_GET_MONOFLOP = 4;
		private static byte CALLBACK_MONOFLOP_DONE = 5;


		/// <summary>
		///  This callback is triggered whenever a monoflop timer reaches 0. The 
		///  parameter contain the relay (1 or 2) and the current state of the relay 
		///  (the state after the monoflop).
		///  
		///  .. versionadded:: 1.1.1~(Plugin)
		/// </summary>
		public delegate void MonoflopDone(byte relay, bool state);

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickletDualRelay(string uid) : base(uid) 
		{
			this.expectedName = "Dual Relay Bricklet";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 1;
			messageCallbacks[CALLBACK_MONOFLOP_DONE] = new MessageCallback(CallbackMonoflopDone);
		}

		/// <summary>
		///  Sets the state of the relays, *true* means on and *false* means off. 
		///  For example: (true, false) turns relay 1 on and relay 2 off.
		///  
		///  If you just want to set one of the relays and don't know the current state
		///  of the other relay, you can get the state with <see cref="Tinkerforge.BrickletDualRelay.GetState"/>.
		///  
		///  Running monoflop timers will be overwritten if this function is called.
		///  
		///  The default value is (false, false).
		/// </summary>
		public void SetState(bool relay1, bool relay2)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_STATE, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(relay1, 4, data_);
			LEConverter.To(relay2, 5, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the state of the relays, *true* means on and *false* means off.
		/// </summary>
		public void GetState(out bool relay1, out bool relay2)
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_STATE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_STATE, out response);

			relay1 = LEConverter.BoolFrom(4, response);
			relay2 = LEConverter.BoolFrom(5, response);
		}

		/// <summary>
		///  The first parameter can be 1 or 2 (relay 1 or relay 2). The second parameter 
		///  is the desired state of the relay (*true* means on and *false* means off).
		///  The third parameter indicates the time (in ms) that the relay should hold 
		///  the state.
		///  
		///  If this function is called with the parameters (1, true, 1500):
		///  Relay 1 will turn on and in 1.5s it will turn off again.
		///  
		///  A monoflop can be used as a failsafe mechanism. For example: Lets assume you 
		///  have a RS485 bus and a Dual Relay Bricklet connected to one of the slave 
		///  stacks. You can now call this function every second, with a time parameter
		///  of two seconds. The relay will be on all the time. If now the RS485 
		///  connection is lost, the relay will turn off in at most two seconds.
		///  
		///  .. versionadded:: 1.1.1~(Plugin)
		/// </summary>
		public void SetMonoflop(byte relay, bool state, uint time)
		{
			byte[] data_ = new byte[10];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_MONOFLOP, 1, data_);
			LEConverter.To((ushort)10, 2, data_);
			LEConverter.To(relay, 4, data_);
			LEConverter.To(state, 5, data_);
			LEConverter.To(time, 6, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns (for the given relay) the current state and the time as set by 
		///  <see cref="Tinkerforge.BrickletDualRelay.SetMonoflop"/> as well as the remaining time until the state flips.
		///  
		///  If the timer is not running currently, the remaining time will be returned
		///  as 0.
		///  
		///  .. versionadded:: 1.1.1~(Plugin)
		/// </summary>
		public void GetMonoflop(byte relay, out bool state, out uint time, out uint timeRemaining)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_MONOFLOP, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(relay, 4, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_MONOFLOP, out response);

			state = LEConverter.BoolFrom(4, response);
			time = LEConverter.UIntFrom(5, response);
			timeRemaining = LEConverter.UIntFrom(9, response);
		}

		internal int CallbackMonoflopDone(byte[] data_)
		{
			byte relay = LEConverter.ByteFrom(4, data_);
			bool state = LEConverter.BoolFrom(5, data_);

			((MonoflopDone)callbacks[CALLBACK_MONOFLOP_DONE])(relay, state);
			return 6;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(MonoflopDone))
			{
				callbacks[CALLBACK_MONOFLOP_DONE] = d;
			}
		}
	}
}
