/* ***********************************************************
 * This file was automatically generated on 2012-10-12.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

using System;

namespace Tinkerforge
{
	/// <summary>
	///  Device for controlling up to 4 optically coupled digital outputs
	/// </summary>
	public class BrickletIndustrialDigitalOut4 : Device 
	{
		private static byte FUNCTION_SET_VALUE = 1;
		private static byte FUNCTION_GET_VALUE = 2;
		private static byte FUNCTION_SET_MONOFLOP = 3;
		private static byte FUNCTION_GET_MONOFLOP = 4;
		private static byte FUNCTION_SET_GROUP = 5;
		private static byte FUNCTION_GET_GROUP = 6;
		private static byte FUNCTION_GET_AVAILABLE_FOR_GROUP = 7;
		private static byte CALLBACK_MONOFLOP_DONE = 8;


		/// <summary>
		///  This callback is triggered whenever a monoflop timer reaches 0. The
		///  parameters contain the involved pins and the current value of the pins
		///  (the value after the monoflop).
		/// </summary>
		public delegate void MonoflopDone(ushort pinMask, ushort valueMask);

		/// <summary>
		///  Creates an object with the unique device ID <c>uid</c>. This object can
		///  then be added to the IP connection.
		/// </summary>
		public BrickletIndustrialDigitalOut4(string uid) : base(uid) 
		{
			this.expectedName = "Industrial Digital Out 4 Bricklet";
			this.bindingVersion[0] = 1;
			this.bindingVersion[1] = 0;
			this.bindingVersion[2] = 0;
			messageCallbacks[CALLBACK_MONOFLOP_DONE] = new MessageCallback(CallbackMonoflopDone);
		}

		/// <summary>
		///  Sets the output value with a bitmask. The bitmask
		///  is 16 bit long, *true* refers to high and *false* refers to 
		///  low.
		///  
		///  For example: The value 0b0000000000000011 will turn pins 0-1 
		///  high and the other pins low.
		///  
		///  If no groups are used (see <see cref="Tinkerforge.BrickletIndustrialDigitalOut4.SetGroup"/>), the pins correspond to the
		///  markings on the Digital Out 4 Bricklet.
		///  
		///  If groups are used, the pins correspond to the element in the group.
		///  Element 1 in the group will get pins 0-3, element 2 pins 4-7, element 3
		///  pins 8-11 and element 4 pins 12-15.
		/// </summary>
		public void SetValue(ushort valueMask)
		{
			byte[] data_ = new byte[6];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_VALUE, 1, data_);
			LEConverter.To((ushort)6, 2, data_);
			LEConverter.To(valueMask, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the bitmask as set by <see cref="Tinkerforge.BrickletIndustrialDigitalOut4.SetValue"/>.
		/// </summary>
		public ushort GetValue()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_VALUE, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_VALUE, out response);

			return LEConverter.UShortFrom(4, response);
		}

		/// <summary>
		///  Configures a monoflop of the pins specified by the first parameter
		///  bitmask.
		///  
		///  The second parameter is a bitmask with the desired value of the specified
		///  pins (*true* means high and *false* means low).
		///  
		///  The third parameter indicates the time (in ms) that the pins should hold
		///  the value.
		///  
		///  If this function is called with the parameters 
		///  ((1 &lt;&lt; 0) | (1 &lt;&lt; 3), (1 &lt;&lt; 0), 1500):
		///  Pin 0 will get high and pin 3 will get low. In 1.5s pin 0 will get low and
		///  pin 3 will get high again.
		///  
		///  A monoflop can be used as a fail-safe mechanism. For example: Lets assume you
		///  have a RS485 bus and a Digital Out 4 Bricklet connected to one of the slave
		///  stacks. You can now call this function every second, with a time parameter
		///  of two seconds and pin 0 high. Pin 0 will be high all the time. If now
		///  the RS485 connection is lost, then pin 0 will turn low in at most two seconds.
		/// </summary>
		public void SetMonoflop(ushort pinMask, ushort valueMask, uint time)
		{
			byte[] data_ = new byte[12];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_MONOFLOP, 1, data_);
			LEConverter.To((ushort)12, 2, data_);
			LEConverter.To(pinMask, 4, data_);
			LEConverter.To(valueMask, 6, data_);
			LEConverter.To(time, 8, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns (for the given pin) the current value and the time as set by
		///  <see cref="Tinkerforge.BrickletIndustrialDigitalOut4.SetMonoflop"/> as well as the remaining time until the value flips.
		///  
		///  If the timer is not running currently, the remaining time will be returned
		///  as 0.
		/// </summary>
		public void GetMonoflop(byte pin, out ushort value, out uint time, out uint timeRemaining)
		{
			byte[] data_ = new byte[5];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_MONOFLOP, 1, data_);
			LEConverter.To((ushort)5, 2, data_);
			LEConverter.To(pin, 4, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_MONOFLOP, out response);

			value = LEConverter.UShortFrom(4, response);
			time = LEConverter.UIntFrom(6, response);
			timeRemaining = LEConverter.UIntFrom(10, response);
		}

		/// <summary>
		///  Sets a group of Digital Out 4 Bricklets that should work together. You can
		///  find Bricklets that can be grouped together with <see cref="Tinkerforge.BrickletIndustrialDigitalOut4.GetAvailableForGroup"/>.
		///  
		///  The group consists of 4 elements. Element 1 in the group will get pins 0-3,
		///  element 2 pins 4-7, element 3 pins 8-11 and element 4 pins 12-15.
		///  
		///  Each element can either be one of the ports ('a' to 'd') or 'n' if it should
		///  not be used.
		///  
		///  For example: If you have two Digital Out 4 Bricklets connected to port A and
		///  port B respectively, you could call with "['a', 'b', 'n', 'n']".
		///  
		///  Now the pins on the Digital Out 4 on port A are assigned to 0-3 and the
		///  pins on the Digital Out 4 on port B are assigned to 4-7. It is now possible
		///  to call <see cref="Tinkerforge.BrickletIndustrialDigitalOut4.SetValue"/> and control two Bricklets at the same time.
		/// </summary>
		public void SetGroup(char[] group)
		{
			byte[] data_ = new byte[8];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_SET_GROUP, 1, data_);
			LEConverter.To((ushort)8, 2, data_);
			LEConverter.To(group, 4, 4, data_);

			SendRequestNoResponse(data_);
		}

		/// <summary>
		///  Returns the group as set by <see cref="Tinkerforge.BrickletIndustrialDigitalOut4.SetGroup"/>
		/// </summary>
		public char[] GetGroup()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_GROUP, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_GROUP, out response);

			return LEConverter.CharArrayFrom(4, response, 4);
		}

		/// <summary>
		///  Returns a bitmask of ports that are available for grouping. For example the
		///  value 0b0101 means: Port *A* and Port *C* are connected to Bricklets that
		///  can be grouped together.
		/// </summary>
		public byte GetAvailableForGroup()
		{
			byte[] data_ = new byte[4];
			LEConverter.To(stackID, 0, data_);
			LEConverter.To(FUNCTION_GET_AVAILABLE_FOR_GROUP, 1, data_);
			LEConverter.To((ushort)4, 2, data_);

			byte[] response;
			SendRequestExpectResponse(data_, FUNCTION_GET_AVAILABLE_FOR_GROUP, out response);

			return LEConverter.ByteFrom(4, response);
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetValue(out ushort valueMask)
		{
			valueMask = GetValue();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetGroup(out char[] group)
		{
			group = GetGroup();
		}

		/// <summary>
		///  Obsolete. Use overloaded version instead that returns the result.
		/// </summary>
		[Obsolete()]
		public void GetAvailableForGroup(out byte available)
		{
			available = GetAvailableForGroup();
		}

		internal int CallbackMonoflopDone(byte[] data_)
		{
			ushort pinMask = LEConverter.UShortFrom(4, data_);
			ushort valueMask = LEConverter.UShortFrom(6, data_);

			((MonoflopDone)callbacks[CALLBACK_MONOFLOP_DONE])(pinMask, valueMask);
			return 8;
		}

		/// <summary>
		///  Registers a callback function.
		/// </summary>
		public void RegisterCallback(System.Delegate d)
		{
			if(d.GetType() == typeof(MonoflopDone))
			{
				callbacks[CALLBACK_MONOFLOP_DONE] = d;
			}
		}
	}
}
